<?php

/**
 * @file
 *
 * @deprecated in drupal:11.4.0 and is removed from drupal:12.0.0. Use \Drupal::service(\Drupal\dblog\DbLogFilters::class)->filters() instead.
 * @see https://www.drupal.org/node/3560399
 */

@trigger_error(__FILE__ . ' is deprecated in drupal:11.4.0 and is removed from drupal:12.0.0. Use \Drupal::service(\Drupal\dblog\DbLogFilters::class)->filters() instead. See https://www.drupal.org/node/3560399', E_USER_DEPRECATED);
